-- Agrega campos requeridos para control de notas de credito
ALTER TABLE tbl_devoluciones
   ADD COLUMN referencia VARCHAR(20) DEFAULT '' AFTER id_devolucion,
   ADD COLUMN aplicado DATETIME AFTER id_usuario_devolucion,
   ADD COLUMN consolidado DATETIME AFTER aplicado;


DROP PROCEDURE IF EXISTS stp_articulosventa;
DROP PROCEDURE IF EXISTS stp_precioProductoNew;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_buscarProducto`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_buscarProducto`(
	in_info TEXT
)
    COMMENT 'Busca un producto por descripción, nombre, modelo o clave'
BEGIN
	SET in_info = CONCAT('%', in_info, '%');
	
	SELECT prd.*
	FROM cat_productos_servicios AS prd
	WHERE prd.clave LIKE in_info
		OR prd.codigo LIKE in_info
		OR prd.modelo LIKE in_info
		OR prd.descripcion LIKE in_info
		OR prd.nombre LIKE in_info;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_cobroPedido`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_cobroPedido`(
	in_id_movimiento INT,
	in_cobro DECIMAL(15,2),
	in_cambio DECIMAL(15,2),
	in_descuento DECIMAL(15,2),
	in_forma_pago INT,
	in_factura BOOL,
	in_id_user INT,
	in_id_user_descuento INT,
	in_referencia VARCHAR(20)
)
    SQL SECURITY INVOKER
    COMMENT 'Aplica el cobro a un pedido'
BEGIN
	UPDATE tbl_movimientos
	SET pagado = pagado + in_cobro - in_cambio + in_descuento,
		descuento = descuento + in_descuento,
		fecha_pago = NOW(),
		factura = in_factura,
		id_usuario_pago = in_id_user
	WHERE id_movimiento = in_id_movimiento;
	
	INSERT INTO tbl_movimientos_pagos
	SET id_movimiento = in_id_movimiento,
		importe_cobro = in_cobro,
		cambio = in_cambio,
		descuento = in_descuento,
		id_forma_pago = in_forma_pago,
		fecha_pago = NOW(),
		id_usuario_pago = in_id_user,
		id_usuario_descuento = in_id_user_descuento;
	
	IF TRIM(in_referencia) <> '' THEN
		UPDATE tbl_devoluciones
		SET aplicado = NOW()
		WHERE referencia = in_referencia;
	END IF;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_configuracion`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_configuracion`()
    SQL SECURITY INVOKER
    COMMENT 'Obtiene la información de configuración'
BEGIN
	SELECT cfg.id_sucursal,
		cfg.default_image,
		cfg.group_len,
		cfg.desglosa_total,
		cfg.cashdrawer_unc,
		cfg.url_existencias,
		cfg.url_sync_empresa,
		cfg.url_sync_sucursales,
		cfg.url_sync_productos,
		cfg.url_sync_precios,
		cfg.url_sync_usuarios,
		cfg.url_sync_ventas,
		cfg.token,
		COALESCE(cfg.logo, '') AS logo,
		cfg.print_hoffset
	FROM cat_configuracion AS cfg;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_efectivoEnCaja`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_efectivoEnCaja`()
    SQL SECURITY INVOKER
    COMMENT 'Obtiene el importe de efectivo en caja actualmente'
BEGIN
	DECLARE efectivo DECIMAL(15,2);
	DECLARE retiros DECIMAL(15,2);
	DECLARE pagos DECIMAL(15,2);
	DECLARE fecha_corte DATETIME;
	SET efectivo = 0;
	SET retiros = 0;
	SET pagos = 0;
	SET fecha_corte = '2000-01-01';
	
	
	SELECT cc.efectivo_real, cc.fecha_corte INTO efectivo, fecha_corte
	FROM tbl_cortes_caja AS cc
	ORDER BY cc.fecha_corte DESC
	LIMIT 1;
	
	
	SELECT SUM(re.retiro) INTO retiros
	FROM tbl_retiros AS re
	WHERE re.fecha_retiro >= fecha_corte;
	
	
	SELECT SUM(p.importe_cobro - p.cambio) INTO pagos
	FROM tbl_movimientos_pagos AS p
	WHERE p.fecha_pago >= fecha_corte
		AND p.id_forma_pago = 0;
	
	
	SELECT efectivo + COALESCE(pagos, 0) - COALESCE(retiros, 0) AS efectivo,
		efectivo AS caja, COALESCE(pagos, 0) AS pagos, COALESCE(retiros, 0) AS retiros;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_existeCorteCaja`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_existeCorteCaja`(
)
    SQL SECURITY INVOKER
    COMMENT 'Verifica si existe corte de caja inicial del dia'
BEGIN
	DECLARE hoy DATE;
	DECLARE existe INT;
	SET hoy = CAST(NOW() AS DATE);
	
	
	SELECT cc.id_corte INTO existe
	FROM tbl_cortes_caja AS cc
	WHERE CAST(cc.fecha_corte AS DATE) = hoy
	ORDER BY cc.fecha_corte DESC
	LIMIT 1;
	
	SELECT IF(ISNULL(existe), 0, 1) AS existe;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_infoVenta`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_infoVenta`(
	in_id_movimiento INT
)
    COMMENT 'Recupera la información del movimiento'
BEGIN
	SELECT mov.*,
		u.nombre AS nombre_vendedor
	FROM tbl_movimientos AS mov
	JOIN tbl_usuarios AS u
		ON u.id_user = mov.id_usuario_venta
	WHERE mov.id_movimiento = in_id_movimiento;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_insertarCorteCaja`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_insertarCorteCaja`(
	in_efectivo_caja DECIMAL(15,2),
	in_efectivo_real DECIMAL(15,2),
	in_id_user_caja INT,
	in_id_user_corte INT,
	in_desglose TEXT
)
    COMMENT 'Agregar registro de corte de caja'
BEGIN
	SET @fecStr = DATE_FORMAT(CURDATE(), '%Y%m%d');
	
	START TRANSACTION;
	SET @consecutivo = (
		SELECT COALESCE(MAX(consecutivo), 0) + 1
		FROM tbl_cortes_caja
		WHERE CAST(fecha_corte AS DATE) = CURDATE()
	);
	
	
	INSERT INTO tbl_cortes_caja
	SET fecha_corte = NOW(),
		consecutivo = @consecutivo,
		efectivo_caja = in_efectivo_caja,
		efectivo_real = in_efectivo_real,
		diferencia = efectivo_real - efectivo_caja,
		id_usuario_caja = in_id_user_caja,
		id_usuario_corte = in_id_user_corte,
		desglose = in_desglose;
	
	SET @id = LAST_INSERT_ID();
	COMMIT;
	
	
	SELECT @id AS id_corte;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_insertarDevolucion`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_insertarDevolucion`(
	in_id_venta INT,
	in_id_producto INT,
	in_cantidad INT,
	in_precio_unitario DECIMAL(15,2),
	in_id_user_caja INT,
	in_id_user_devolucion INT
)
    COMMENT 'Agregar registro de devolución de producto'
BEGIN
    SET @ref = CONCAT(CHAR(65 + FLOOR( 1 + RAND( ) * 25)), '-', LPAD(FLOOR( 1 + RAND( ) * 999999), 6, "0"));
    
    INSERT INTO tbl_devoluciones
    SET fecha_devolucion = NOW(),
        id_venta = in_id_venta,
        referencia = @ref,
        id_producto = in_id_producto,
        cantidad = in_cantidad,
        precio_unitario = in_precio_unitario,
        devolucion = precio_unitario * cantidad,
        id_usuario_caja = in_id_user_caja,
        id_usuario_devolucion = in_id_user_devolucion;
    
    SET @id = LAST_INSERT_ID();
    
    
    SELECT  @id AS id_devolucion;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_insertarMovimiento`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_insertarMovimiento`(
	in_id_movimiento INT,
	in_id_sucursal INT,
	in_id_tipo_movimiento INT,
	in_subtotal DECIMAL(15,2),
	in_iva	DECIMAL(15,2),
	in_total DECIMAL(15,2),
	in_id_user INT
)
    COMMENT 'Guardar (INS/UPD) movimiento en la tabla de movimientos'
BEGIN
	SET @ref = CONCAT(CHAR(65 + FLOOR( 1 + RAND( ) * 25)), '-', LPAD(FLOOR( 1 + RAND( ) * 999999), 6, "0"));
	SET @hora = NOW();
	
	START TRANSACTION;
	SET @consecutivo = (
		SELECT COALESCE(MAX(consecutivo), 0) + 1
		FROM tbl_movimientos 
		WHERE fecha = CAST(@hora AS DATE)
	);
	
	INSERT INTO tbl_movimientos
	SET id_movimiento = in_id_movimiento,
		id_sucursal = in_id_sucursal,
		id_tipo_movimiento = in_id_tipo_movimiento,
		consecutivo = @consecutivo,
		referencia = @ref,
		hora = SUBSTR(@hora, 12, 8),
		fecha = CAST(@hora AS DATE),
		subtotal = in_subtotal,
		iva = in_iva,
		total = in_total,
		pagado = 0,
		id_usuario_venta = in_id_user
	ON DUPLICATE KEY
	UPDATE id_sucursal = in_id_sucursal,
		id_tipo_movimiento = in_id_tipo_movimiento,
		
		
		hora = SUBSTR(@hora, 12, 8),
		fecha = CAST(@hora AS DATE),
		subtotal = in_subtotal,
		iva = in_iva,
		total = in_total;
		
	
	SET @id = COALESCE(in_id_movimiento, LAST_INSERT_ID());
	
	
	DELETE FROM tbl_movimientos_detalle
	WHERE id_movimiento = @id;
	
	COMMIT;
	
	
	SELECT  mov.id_movimiento, mov.referencia
	FROM tbl_movimientos AS mov
	WHERE mov.id_movimiento = @id;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_insertarRetiro`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_insertarRetiro`(
	in_inicial DECIMAL(15,2),
	in_retiro DECIMAL(15,2),
	in_id_user_caja INT,
	in_id_user_retiro INT
)
    COMMENT 'Agregar registro de retiro de caja'
BEGIN
	START TRANSACTION;
	SET @consecutivo = (
		SELECT COALESCE(MAX(consecutivo), 0) + 1
		FROM tbl_retiros
		WHERE CAST(fecha_retiro AS DATE) = CURDATE()
	);
	INSERT INTO tbl_retiros
	SET fecha_retiro = NOW(),
		consecutivo = @consecutivo,
		efectivo_inicial = in_inicial,
		retiro = in_retiro,
		efectivo_final = efectivo_inicial - retiro,
		id_usuario_caja = in_id_user_caja,
		id_usuario_retiro = in_id_user_retiro;
	
	SET @id = LAST_INSERT_ID();
	COMMIT;
	
	
	SELECT  @id AS id_retiro;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_listarPedidos`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_listarPedidos`(
	in_tipo_movimiento INT,
	in_fecha DATE
)
    COMMENT 'Recupera los pedidos que no se han pagado aun'
BEGIN
	CASE in_tipo_movimiento
		WHEN 1 THEN
			
			SELECT mov.id_movimiento,
				mov.consecutivo,
				mov.fecha,
				mov.hora,
				CONCAT(IF(ISNULL(mov.fecha_cancelado), '', 'CANCELADO - '), mov.referencia) AS referencia,
				u.nombre AS nombre_vendedor,
				mov.total,
				mov.total - mov.pagado AS saldo,
				IF(ISNULL(mov.consolidado), '', 'OK') AS txt_enviada,
				1 AS activo,
				IF(ISNULL(mov.consolidado), 0, 1) AS enviado,
				mov.pagado,
				GROUP_CONCAT(ELT(mp.id_forma_pago + 1,'Efectivo','T. Débito','T. Crédito','Nota Crédito')) AS forma_pago,
				IF(mov.factura = 1, 'SI', '') AS factura
			FROM tbl_movimientos AS mov
			JOIN tbl_usuarios AS u
				ON u.id_user = mov.id_usuario_venta
			LEFT OUTER JOIN tbl_movimientos_pagos AS mp
				ON mp.id_movimiento = mov.id_movimiento
			WHERE mov.pagado <> mov.total
				AND mov.fecha_cancelado IS NULL
			GROUP BY mov.id_movimiento;
		WHEN 2 THEN
			
			SELECT mov.id_movimiento,
				mov.consecutivo,
				mov.fecha,
				mov.hora,
				CONCAT(IF(ISNULL(mov.fecha_cancelado), '', 'CANCELADO - '), mov.referencia) AS referencia,
				u.nombre AS nombre_vendedor,
				mov.total,
				mov.total - mov.pagado AS saldo,
				IF(ISNULL(mov.consolidado), '', 'OK') AS txt_enviada,
				0 AS activo,
				IF(ISNULL(mov.consolidado), 0, 1) AS enviado,
				mov.pagado,
				GROUP_CONCAT(ELT(mp.id_forma_pago + 1,'Efectivo','T. Débito','T. Crédito','Nota Crédito')) AS forma_pago,
				IF(mov.factura = 1, 'SI', '') AS factura
			FROM tbl_movimientos AS mov
			JOIN tbl_usuarios AS u
				ON u.id_user = mov.id_usuario_venta
			LEFT OUTER JOIN tbl_movimientos_pagos AS mp
				ON mp.id_movimiento = mov.id_movimiento
			WHERE mov.pagado = mov.total
				AND mov.fecha = in_fecha
			GROUP BY mov.id_movimiento;
		WHEN 3 THEN
			
			SELECT mov.id_movimiento,
				mov.consecutivo,
				mov.fecha,
				mov.hora,
				CONCAT(IF(ISNULL(mov.fecha_cancelado), '', 'CANCELADO - '), mov.referencia) AS referencia,
				u.nombre AS nombre_vendedor,
				mov.total,
				mov.total - mov.pagado AS saldo,
				IF(ISNULL(mov.consolidado), '', 'OK') AS txt_enviada,
				0 AS activo,
				IF(ISNULL(mov.consolidado), 0, 1) AS enviado,
				mov.pagado,
				GROUP_CONCAT(ELT(mp.id_forma_pago + 1,'Efectivo','T. Débito','T. Crédito','Nota Crédito')) AS forma_pago,
				IF(mov.factura = 1, 'SI', '') AS factura
			FROM tbl_movimientos AS mov
			JOIN tbl_usuarios AS u
				ON u.id_user = mov.id_usuario_venta
			LEFT OUTER JOIN tbl_movimientos_pagos AS mp
				ON mp.id_movimiento = mov.id_movimiento
			WHERE mov.fecha_cancelado IS NOT NULL
				AND mov.fecha = in_fecha
			GROUP BY mov.id_movimiento;
		WHEN 4 THEN
			
			SELECT mov.id_movimiento,
				mov.consecutivo,
				mov.fecha,
				mov.hora,
				CONCAT(IF(ISNULL(mov.fecha_cancelado), '', 'CANCELADO - '), mov.referencia) AS referencia,
				u.nombre AS nombre_vendedor,
				mov.total,
				mov.total - mov.pagado AS saldo,
				IF(ISNULL(mov.consolidado), '', 'OK') AS txt_enviada,
				IF(mov.pagado <> mov.total AND ISNULL(mov.fecha_cancelado), 1, 0) AS activo,
				IF(ISNULL(mov.consolidado), 0, 1) AS enviado,
				mov.pagado,
				GROUP_CONCAT(ELT(mp.id_forma_pago + 1,'Efectivo','T. Débito','T. Crédito','Nota Crédito')) AS forma_pago,
				IF(mov.factura = 1, 'SI', '') AS factura
			FROM tbl_movimientos AS mov
			JOIN tbl_usuarios AS u
				ON u.id_user = mov.id_usuario_venta
			LEFT OUTER JOIN tbl_movimientos_pagos AS mp
				ON mp.id_movimiento = mov.id_movimiento
			WHERE mov.fecha = in_fecha
			GROUP BY mov.id_movimiento
			; 
	END CASE;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_login`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_login`(
	p_username VARCHAR(20),
	p_password VARCHAR(20)
    )
    COMMENT 'Valida usuario para inicio de sesión'
BEGIN
	SELECT u.id_user,
		u.nombre
	FROM tbl_usuarios AS u
	WHERE BINARY user_name = p_username
		AND BINARY user_pass = p_password;
    END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_notaCreditoPendiente`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_notaCreditoPendiente`(
	in_referencia VARCHAR(20)
)
    COMMENT 'Recupera nota de crédito pendiente por aplicar'
BEGIN
    SELECT dev.*
    FROM tbl_devoluciones AS dev
    WHERE dev.referencia = in_referencia
        AND dev.aplicado IS NULL;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_permisosUsuario`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_permisosUsuario`(
	in_id_user INT
)
    COMMENT 'Informacion de la sucursal'
BEGIN
	SELECT usr.permisos
	FROM tbl_usuarios AS usr
	WHERE usr.id_user = in_id_user;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_precioProducto`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_precioProducto`(
	in_id_sucursal INT, 	
	in_id_producto INT, 	
	in_cantidad INT 	
)
    COMMENT 'Obtiene la imagen por default para productos sin foto'
BEGIN
	SET @precio_unitario = NULL;
	SET @precio_mayoreo = 0.00;
	SET @oferta = 0;
	SET @cantidad = 0;
	
	
	SELECT pof.precio,
		pof.precio,
		1 AS oferta
	INTO @precio_unitario,
		@precio_mayoreo,
		@oferta
	FROM tbl_precios_ofertas AS pof
	WHERE pof.id_producto = in_id_producto
		AND pof.id_sucursal IN (0, in_id_sucursal)
		AND ( pof.fecha_desde IS NULL AND pof.fecha_hasta IS NULL
			OR pof.fecha_desde IS NULL AND CURDATE() <= pof.fecha_hasta
			OR CURDATE() >= pof.fecha_desde AND pof.fecha_hasta IS NULL
			OR CURDATE() BETWEEN pof.fecha_desde AND pof.fecha_hasta
		)
	ORDER BY pof.id_sucursal DESC
	LIMIT 1;
	
	
	IF ISNULL(@precio_unitario) THEN
		
		IF EXISTS (
			SELECT ppro.id_producto
			FROM tbl_precios_productos AS ppro
			WHERE ppro.id_sucursal = in_id_sucursal
				AND ppro.id_producto = in_id_producto
		) THEN
			
			
			
			SELECT COALESCE(pp.precio, prd.precio_venta),
				COALESCE(( 
					SELECT pma.precio
					FROM tbl_precios_mayoreo AS pma
					WHERE pma.id_producto = in_id_producto
						AND pma.id_sucursal IN (0, in_id_sucursal)
					ORDER BY pma.id_sucursal DESC
					LIMIT 1
				), pp.precio, prd.precio_venta),
				COALESCE(pp.cantidad, 0)
			INTO @precio_unitario,
				@precio_mayoreo,
				@cantidad
			FROM cat_productos_servicios AS prd
			JOIN cat_sucursales AS suc
				ON suc.id_sucursal = in_id_sucursal
			LEFT OUTER JOIN tbl_precios_productos AS pp
				ON pp.id_producto = prd.id_producto
					AND pp.id_sucursal = in_id_sucursal
					AND pp.cantidad <= in_cantidad
			WHERE prd.id_producto = in_id_producto
			ORDER BY pp.cantidad DESC
			LIMIT 1;
		ELSE
			
			
			
			
			SELECT COALESCE(pp.precio, prd.precio_venta),
				COALESCE(( 
					SELECT pma.precio
					FROM tbl_precios_mayoreo AS pma
					WHERE pma.id_producto = in_id_producto
						AND pma.id_sucursal IN (0, in_id_sucursal)
					ORDER BY pma.id_sucursal DESC
					LIMIT 1
				), pp.precio, prd.precio_venta),
				COALESCE(pp.cantidad, 0)
			INTO @precio_unitario,
				@precio_mayoreo,
				@cantidad
			FROM cat_productos_servicios AS prd
			JOIN cat_sucursales AS suc
				ON suc.id_sucursal = in_id_sucursal
			LEFT OUTER JOIN tbl_precios_productos AS pp
				ON pp.id_producto = prd.id_producto
					AND pp.id_sucursal = 0 
					AND pp.cantidad <= in_cantidad
			WHERE prd.id_producto = in_id_producto
			ORDER BY pp.cantidad DESC
			LIMIT 1;
		END IF;
	END IF;
	
	SELECT @precio_unitario AS precio_unitario,
		@precio_mayoreo AS precio_mayoreo,
		@cantidad AS cantidad_precio,
		@oferta AS precio_oferta;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_preciosProducto`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_preciosProducto`(
	in_id_sucursal INT,
	in_id_producto INT
)
    SQL SECURITY INVOKER
    COMMENT 'Obtiene la imagen por default para productos sin foto'
BEGIN
	IF EXISTS (
		SELECT id_producto FROM tbl_precios_productos
		WHERE id_sucursal = in_id_sucursal
			AND id_producto = in_id_producto
	) THEN
		
		SELECT pp.cantidad,
			pp.precio
		FROM tbl_precios_productos AS pp
		WHERE pp.id_producto = in_id_producto
			AND pp.id_sucursal = in_id_sucursal
		ORDER BY pp.cantidad;
	ELSE
		
		SELECT pp.cantidad,
			pp.precio
		FROM tbl_precios_productos AS pp
		WHERE pp.id_producto = in_id_producto
			AND pp.id_sucursal = 0
		ORDER BY pp.cantidad;
	END IF;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_producto_x_codigo`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_producto_x_codigo`(
	p_codigo VARCHAR(30)
)
    COMMENT 'Búsqueda de Producto por Código'
BEGIN
	SELECT 1 AS cantidad,
		prd.modelo,
		prd.nombre AS descripcion,
		prd.precio_venta AS precio_unitario,
		prd.precio_venta,
		0.00 AS subtotal,
		0.00 AS iva,
		0.00 AS total,
		prd.tasa_iva,
		prd.iva_incluido,
		prd.imagen,
		prd.id_producto,
		prd.clave
	FROM cat_productos_servicios AS prd
	JOIN cat_configuracion AS cfg
	WHERE BINARY prd.clave = p_codigo
		OR BINARY LEFT(prd.clave, cfg.group_len) = p_codigo
		OR BINARY prd.modelo = p_codigo;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_requiereCorteCaja`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_requiereCorteCaja`(
	p_id_user INT
)
    SQL SECURITY INVOKER
    COMMENT 'Verifica si usuario que incia sesion requiere generar corte'
BEGIN
	DECLARE idUsuarioUltimoCorte INT;
	DECLARE hoy DATE;
	SET idUsuarioUltimoCorte = 0;
	SET hoy = CAST(NOW() AS DATE);
	
	SELECT cc.id_usuario_caja INTO idUsuarioUltimoCorte
	FROM tbl_cortes_caja AS cc
	WHERE CAST(cc.fecha_corte AS DATE) = hoy
	ORDER BY cc.fecha_corte DESC
	LIMIT 1;
	
	
	SELECT IF(p_id_user <> COALESCE(idUsuarioUltimoCorte, 0), 0, 1) AS existe_corte;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_syncSucursal`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_syncSucursal`(
	in_id_sucursal INT
)
    SQL SECURITY INVOKER
    COMMENT 'Extraer información de sucursal y matriz para sincronización'
BEGIN
	SELECT suc.*
	FROM cat_sucursales AS suc
	WHERE suc.id_sucursal = in_id_sucursal
		OR suc.matriz = 1;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_syncVentaDetail`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_syncVentaDetail`(
	in_id_movimiento INT
)
    COMMENT 'Recupera los registros del detalle de una venta'
BEGIN
	SELECT dm.id_producto,
		p.id_unidad_medida,
		CAST(dm.cantidad AS UNSIGNED) AS cantidad,
		dm.precio_venta
	FROM tbl_movimientos_detalle AS dm
	JOIN cat_productos_servicios AS p
		ON p.id_producto = dm.id_producto
	WHERE dm.id_movimiento = in_id_movimiento;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_syncVentas`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_syncVentas`(
)
    COMMENT 'Recupera la sig. venta pendiente de sincronizar'
BEGIN
    SELECT m.id_movimiento
    FROM tbl_movimientos AS m
    WHERE m.id_tipo_movimiento = 1
        AND m.pagado >= total
        AND m.fecha_cancelado IS NULL
        AND m.consolidado IS NULL
    ORDER BY m.fecha_pago
    LIMIT 1;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_updateConfig`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_updateConfig`(
	in_desglosa_total INT,
	in_cashdrawer_unc TEXT,
	in_token VARCHAR(20),
	in_sucursal INT,
	in_offset INT
)
    COMMENT 'Guarda las opciones de configuración general'
BEGIN
	UPDATE cat_configuracion
	SET desglosa_total = in_desglosa_total,
		cashdrawer_unc = in_cashdrawer_unc,
		token = in_token,
		id_sucursal = in_sucursal,
		print_hoffset = in_offset;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_updateEmpresa`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_updateEmpresa`(
	in_logo TEXT
)
    COMMENT 'Guarda la información de la empresa'
BEGIN
	UPDATE cat_configuracion
	SET logo = in_logo;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_updateSync`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_updateSync`(
	in_tipo VARCHAR(20)
)
    COMMENT 'Actualiza la fecha ultima sincronización por tipo'
BEGIN
	CASE in_tipo
		WHEN 'sucursales' THEN
			UPDATE tbl_sincronizacion
			SET sync_sucursales = NOW();
		WHEN 'productos' THEN
			UPDATE tbl_sincronizacion
			SET sync_productos = NOW();
		WHEN 'precios' THEN
			UPDATE tbl_sincronizacion
			SET sync_precios = NOW();
		WHEN 'usuarios' THEN
			UPDATE tbl_sincronizacion
			SET sync_usuarios = NOW();
	END CASE;
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_updateSyncVenta`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_updateSyncVenta`(
	in_id_venta INT
)
    COMMENT 'Marca la venta como sincronizada'
BEGIN
	UPDATE tbl_movimientos
	SET consolidado = NOW()
	WHERE id_movimiento = in_id_venta;
	
	UPDATE tbl_sincronizacion
	SET sync_ventas = NOW();
END$$

DELIMITER ;


DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_validarAutorizacion`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_validarAutorizacion`(
	in_usuario VARCHAR(40),
	in_password VARCHAR(40),
	in_permiso VARCHAR(80)
)
    COMMENT 'Verifica usuario con contraseña y permiso'
BEGIN
	SELECT u.id_user
	FROM tbl_usuarios AS u
	WHERE BINARY user_name = in_usuario
		AND BINARY user_pass = in_password
		AND INSTR(u.permisos, in_permiso) > 0
		AND u.activo = 1;
END$$

DELIMITER ;